//
//  MyProfileTableViewController.swift
//  Tusker
//
//  Created by Shadowfacts on 11/24/18.
//  Copyright © 2018 Shadowfacts. All rights reserved.
//

import UIKit
import SwiftUI

class MyProfileTableViewController: ProfileTableViewController {

    init() {
        super.init(accountID: nil)
        
        title = "My Profile"
        tabBarItem.image = UIImage(systemName: "person.fill")
        
        
        MastodonController.getOwnAccount { (account) in
            self.accountID = account.id
            
            ImageCache.avatars.get(account.avatar, completion: { (data) in
                guard let data = data else { return }
                DispatchQueue.main.async {
                    let image = UIImage(data: data)!
                    let size = CGSize(width: 30, height: 30)
                    let tabBarImage = UIGraphicsImageRenderer(size: size).image { (_) in
                        image.draw(in: CGRect(origin: .zero, size: size))
                    }
                    let alwaysOriginalImage = tabBarImage.withRenderingMode(.alwaysOriginal)
                    self.tabBarItem.image = alwaysOriginalImage
                }
            })
        }
        
        navigationItem.rightBarButtonItem = UIBarButtonItem(title: "Preferences", style: .plain, target: self, action: #selector(preferencesPressed))
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
    }
    
    @objc func preferencesPressed() {
        let view = PreferencesView().environmentObject(Preferences.shared)
        let hostingController = UIHostingController(rootView: view)
        let navigationController = UINavigationController(rootViewController: hostingController)
        hostingController.navigationItem.rightBarButtonItem = UIBarButtonItem(barButtonSystemItem: .done, target: self, action: #selector(closePreferences))
        present(navigationController, animated: true)
    }
    
    @objc func closePreferences() {
        dismiss(animated: true)
    }

}
