//
//  XCBManager.swift
//  Tusker
//
//  Created by Shadowfacts on 9/23/18.
//  Copyright © 2018 Shadowfacts. All rights reserved.
//

import UIKit

class XCBManager {
    
    static var specs: [XCBRequestSpec] = [
        // Statuses
        XCBRequestSpec(type: .showStatus, arguments: ["statusID": true, "statusURL": true], canRunSilently: false, action: XCBActions.showStatus),
        XCBRequestSpec(type: .getStatus, arguments: ["statusID": true, "statusURL": true, "html": true], canRunSilently: false, action: XCBActions.getStatus),
        XCBRequestSpec(type: .postStatus, arguments: ["mentioning": true, "text": true], canRunSilently: true, action: XCBActions.postStatus),
        XCBRequestSpec(type: .favoriteStatus, arguments: ["statusID": true, "statusURL": true], canRunSilently: true, action: XCBActions.favoriteStatus),
        XCBRequestSpec(type: .reblogStatus, arguments: ["statusID": true, "statusURL": true], canRunSilently: true, action: XCBActions.reblogStatus),
        // Accounts
        XCBRequestSpec(type: .showAccount, arguments: ["accountID": true, "accountURL": true, "acct": true], canRunSilently: false, action: XCBActions.showAccount),
        XCBRequestSpec(type: .getAccount, arguments: ["accountID": true, "accountURL": true, "acct": true], canRunSilently: false, action: XCBActions.getAccount),
        XCBRequestSpec(type: .getCurrentUser, arguments: [:], canRunSilently: false, action: XCBActions.getCurrentUser),
        XCBRequestSpec(type: .followUser, arguments: ["accountID": true, "accountURL": true, "acct": true], canRunSilently: true, action: XCBActions.followUser),
        // Search
        XCBRequestSpec(type: .search, arguments: ["query": false], canRunSilently: false, action: XCBActions.search),
    ]
    
    static var currentSession: XCBSession?
    
    static func handle(url: URL) -> Bool {
        guard let components = URLComponents(url: url, resolvingAgainstBaseURL: true) else { return false }
        if let spec = specs.first(where: { $0.matches(components) }) {
            let request = XCBRequest(spec: spec, components: components)
            return spec.handle(request: request)
        }
        return false
    }
    
    static func createSession(type: XCBActionType, request: XCBRequest) -> XCBSession {
        let session = XCBSession(type: type, request: request)
        currentSession = session
        return session
    }
    
}
